'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var ReactDOM = _interopDefault(require('react-dom'));
var ExecutionEnvironment = _interopDefault(require('exenv'));
var is = _interopDefault(require('@sindresorhus/is'));
var scroll = _interopDefault(require('scroll'));
var scrollDoc = _interopDefault(require('scroll-doc'));
var deepmerge = _interopDefault(require('deepmerge'));
var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));
var treeChanges = _interopDefault(require('tree-changes'));
var isRequiredIf = _interopDefault(require('react-proptype-conditional-require'));
var ReactTooltips = _interopDefault(require('react-tooltips'));
var deep = _interopDefault(require('deep-diff'));

var STATUS = {
  IDLE: 'idle',
  READY: 'ready',
  WAITING: 'waiting',
  RUNNING: 'running',
  PAUSED: 'paused',
  SKIPPED: 'skipped',
  FINISHED: 'finished',
  ERROR: 'error'
};

var ACTIONS = {
  INIT: 'init',
  START: 'start',
  STOP: 'stop',
  RESET: 'reset',
  RESTART: 'restart',
  PREV: 'prev',
  NEXT: 'next',
  GO: 'go',
  INDEX: 'index',
  CLOSE: 'close',
  SKIP: 'skip',
  UPDATE: 'update'
};

var LIFECYCLE = {
  INIT: 'init',
  READY: 'ready',
  BEACON: 'beacon',
  TOOLTIP: 'tooltip',
  COMPLETE: 'complete',
  ERROR: 'error'
};

var canUseDOM = ExecutionEnvironment.canUseDOM;

var isReact16 = ReactDOM.createPortal !== undefined;

/**
 * Get the current browser
 *
 * @returns {String}
 */
function getBrowser() {
  /* istanbul ignore if */
  if (typeof window === 'undefined') {
    return 'node';
  }

  // Opera 8.0+
  if (Boolean(window.opera) || navigator.userAgent.indexOf(' OPR/') >= 0) {
    return 'opera';
  }
  // Firefox 1.0+
  if (typeof window.InstallTrigger !== 'undefined') {
    return 'firefox';
  }
  // Chrome 1+
  if (!!window.chrome && !!window.chrome.webstore) {
    return 'chrome';
  }

  if (/Version\/([0-9._]+).*Safari/.test(navigator.userAgent)) {
    return 'safari';
  }

  if (document.documentMode) {
    return 'ie';
  }

  return navigator.userAgent;
}

/**
 * Detect legacy browsers
 *
 * @returns {boolean}
 */
function isLegacy() {
  return !(['chrome', 'safari', 'firefox', 'opera'].indexOf(getBrowser()) !== -1);
}

/**
 * Log method calls if debug is enabled
 *
 * @private
 * @param {Object}       arg
 * @param {string}       arg.title    - The title the logger was called from
 * @param {Object|Array} [arg.data]   - The data to be logged
 * @param {boolean}      [arg.warn]  - If true, the message will be a warning
 * @param {boolean}      [arg.debug] - Nothing will be logged unless debug is true
 */
function log(_ref) {
  var title = _ref.title,
      data = _ref.data,
      _ref$warn = _ref.warn,
      warn = _ref$warn === undefined ? false : _ref$warn,
      _ref$debug = _ref.debug,
      debug = _ref$debug === undefined ? false : _ref$debug;

  /* eslint-disable no-console */
  var logFn = warn ? console.warn || console.error : console.log;

  if (debug && title && data) {
    console.groupCollapsed('%creact-joyride: ' + title, 'color: #ff0044; font-weight: bold; font-size: 12px;');

    if (Array.isArray(data)) {
      data.forEach(function (d) {
        if (is.plainObject(d) && d.key) {
          logFn.apply(console, [d.key, d.value]);
        } else {
          logFn.apply(console, [d]);
        }
      });
    } else {
      logFn.apply(console, [data]);
    }

    console.groupEnd();
  }
  /* eslint-enable */
}

function hasValidKeys(value, keys) {
  if (!is.plainObject(value) || !is.array(keys)) return false;
  var validKeys = keys;

  if (is.string(keys)) {
    validKeys = [keys];
  }

  return Object.keys(value).every(function (d) {
    return validKeys.indexOf(d) !== -1;
  });
}

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var defaultState = {
  action: '',
  controlled: false,
  index: 0,
  lifecycle: LIFECYCLE.INIT,
  size: 0,
  status: STATUS.IDLE
};

var validKeys = ['action', 'index', 'lifecycle', 'status'];

function createState(props) {
  var store = new Map();
  var data = new Map();

  var State = function () {
    function State() {
      var _this = this;

      var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          _ref$continuous = _ref.continuous,
          continuous = _ref$continuous === undefined ? false : _ref$continuous,
          stepIndex = _ref.stepIndex,
          _ref$steps = _ref.steps,
          steps = _ref$steps === undefined ? [] : _ref$steps;

      classCallCheck(this, State);

      this.update = function (state) {
        if (!hasValidKeys(state, validKeys)) {
          throw new Error('state is not valid');
        }

        _this.setState(_extends({}, _this.getNextState(_extends({}, _this.getState(), state, {
          action: state.action || ACTIONS.UPDATE
        }), true)));
      };

      this.steps = function (nextSteps) {
        if (!is.array(nextSteps)) return;

        _this.setSteps(nextSteps);
      };

      this.start = function (nextIndex) {
        var _getState = _this.getState(),
            index = _getState.index,
            size = _getState.size;

        _this.setState(_extends({}, _this.getNextState({
          action: ACTIONS.START,
          index: is.number(nextIndex) ? nextIndex : index
        }), {
          status: size ? STATUS.RUNNING : STATUS.WAITING
        }));
      };

      this.stop = function () {
        var advance = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

        var _getState2 = _this.getState(),
            index = _getState2.index;

        _this.setState(_extends({}, _this.getNextState({ action: ACTIONS.STOP, index: index + (advance ? 1 : 0) }), {
          status: STATUS.PAUSED
        }));
      };

      this.restart = function () {
        var _getState3 = _this.getState(),
            controlled = _getState3.controlled;

        if (controlled) return;

        _this.setState(_extends({}, _this.getNextState({ action: ACTIONS.RESTART, index: 0 }), {
          status: STATUS.RUNNING
        }));
      };

      this.reset = function () {
        var _getState4 = _this.getState(),
            controlled = _getState4.controlled;

        if (controlled) return;

        _this.setState(_extends({}, _this.getNextState({ action: ACTIONS.RESET, index: 0 }), {
          status: STATUS.READY
        }));
      };

      this.prev = function () {
        var _getState5 = _this.getState(),
            index = _getState5.index,
            status = _getState5.status;

        if (status !== STATUS.RUNNING) return;

        _this.setState(_extends({}, _this.getNextState({ action: ACTIONS.PREV, index: index - 1 })));
      };

      this.next = function () {
        var _getState6 = _this.getState(),
            index = _getState6.index,
            status = _getState6.status;

        if (status !== STATUS.RUNNING) return;

        _this.setState(_this.getNextState({ action: ACTIONS.NEXT, index: index + 1 }));
      };

      this.go = function (number) {
        var _getState7 = _this.getState(),
            index = _getState7.index,
            status = _getState7.status;

        if (status !== STATUS.RUNNING) return;

        _this.setState(_extends({}, _this.getNextState({ action: ACTIONS.GO, index: index + number })));
      };

      this.index = function (nextIndex) {
        var _getState8 = _this.getState(),
            status = _getState8.status;

        if (status !== STATUS.RUNNING) return;

        var step = _this.getSteps()[nextIndex];

        _this.setState(_extends({}, _this.getNextState({ action: ACTIONS.INDEX, index: nextIndex }), {
          status: step ? status : STATUS.FINISHED
        }));
      };

      this.close = function () {
        var _getState9 = _this.getState(),
            index = _getState9.index,
            status = _getState9.status;

        if (status !== STATUS.RUNNING) return;

        _this.setState(_extends({}, _this.getNextState({ action: ACTIONS.CLOSE, index: index + 1 })));
      };

      this.skip = function () {
        var _getState10 = _this.getState(),
            status = _getState10.status;

        if (status !== STATUS.RUNNING) return;

        _this.setState({
          action: ACTIONS.SKIP,
          lifecycle: LIFECYCLE.INIT,
          status: STATUS.SKIPPED
        });
      };

      this.info = function () {
        return _this.getState();
      };

      this.setState({
        action: ACTIONS.INIT,
        controlled: is.number(stepIndex),
        continuous: continuous,
        index: is.number(stepIndex) ? stepIndex : 0,
        lifecycle: LIFECYCLE.INIT,
        status: steps.length ? STATUS.READY : STATUS.IDLE
      }, true);

      this.setSteps(steps);
    }

    createClass(State, [{
      key: 'addListener',
      value: function addListener(listener) {
        this.listener = listener;
      }
    }, {
      key: 'setState',
      value: function setState(nextState) {
        var initial = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        var state = this.getState();

        var _state$nextState = _extends({}, state, nextState),
            action = _state$nextState.action,
            index = _state$nextState.index,
            lifecycle = _state$nextState.lifecycle,
            status = _state$nextState.status;

        store.set('action', action);
        store.set('index', index);
        store.set('lifecycle', lifecycle);
        store.set('status', status);

        if (initial) {
          store.set('controlled', nextState.controlled);
          store.set('continuous', nextState.continuous);
        }

        /* istanbul ignore else */
        if (this.listener && this.hasUpdatedState(state)) {
          this.listener(this.getState());
        }
      }
    }, {
      key: 'getState',
      value: function getState() {
        if (!store.size) {
          return _extends({}, defaultState);
        }

        var index = parseInt(store.get('index'), 10);
        var steps = this.getSteps();
        var size = steps.length;

        return {
          action: store.get('action'),
          controlled: store.get('controlled'),
          index: index,
          lifecycle: store.get('lifecycle'),
          size: size,
          status: store.get('status')
        };
      }
    }, {
      key: 'getNextState',
      value: function getNextState(state) {
        var force = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        var _getState11 = this.getState(),
            action = _getState11.action,
            controlled = _getState11.controlled,
            index = _getState11.index,
            size = _getState11.size,
            status = _getState11.status;

        var newIndex = is.number(state.index) ? state.index : index;
        var nextIndex = controlled && !force ? index : Math.min(Math.max(newIndex, 0), size);

        return {
          action: state.action || action,
          index: nextIndex,
          lifecycle: state.lifecycle || LIFECYCLE.INIT,
          status: nextIndex === size ? STATUS.FINISHED : state.status || status
        };
      }
    }, {
      key: 'hasUpdatedState',
      value: function hasUpdatedState(oldState) {
        var before = JSON.stringify(oldState);
        var after = JSON.stringify(this.getState());

        return before !== after;
      }
    }, {
      key: 'setSteps',
      value: function setSteps(steps) {
        var _getState12 = this.getState(),
            size = _getState12.size,
            status = _getState12.status;

        data.set('steps', steps);

        if (status === STATUS.WAITING && !size && steps.length) {
          this.setState({ status: STATUS.RUNNING });
        }
      }
    }, {
      key: 'getSteps',
      value: function getSteps() {
        var steps = data.get('steps');

        return Array.isArray(steps) ? steps : [];
      }
    }, {
      key: 'getHelpers',
      value: function getHelpers() {
        return {
          start: this.start,
          stop: this.stop,
          restart: this.restart,
          reset: this.reset,
          prev: this.prev,
          next: this.next,
          go: this.go,
          index: this.index,
          close: this.close,
          skip: this.skip,
          info: this.info
        };
      }
    }]);
    return State;
  }();

  return new State(props);
}

/**
 * Find and return the target DOM element based on a step's 'target'.
 *
 * @private
 * @param {string|HTMLElement} target
 *
 * @returns {HTMLElement|undefined}
 */
function getElement(target) {
  if (typeof target === 'string') {
    return document.querySelector(target);
  }

  return target;
}

/**
 * Get the scroll position.
 *
 * @returns {Number|*|number}
 */
function getScrollTop() {
  return window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
}

/**
 * Find the bounding client rect
 *
 * @private
 * @param {HTMLElement} element - The target element
 * @param {HTMLElement} [offsetParent] - The parent element to calculate offsets from
 * @returns {Object}
 */
function getOffsetBoundingClientRect(element, offsetParent) {
  var elementRect = element.getBoundingClientRect();

  if (!offsetParent) {
    return elementRect;
  }

  var offsetParentRect = offsetParent.getBoundingClientRect();

  var offsetTop = offsetParentRect.top > 0 ? elementRect.top - offsetParentRect.top : elementRect.top;
  var offsetLeft = offsetParentRect.left > 0 ? elementRect.left - offsetParentRect.left : elementRect.left;
  var offsetRight = offsetParentRect.right > 0 ? offsetParentRect.right - elementRect.right : elementRect.right;
  var offsetBottom = offsetParentRect.bottom > 0 ? offsetParentRect.bottom - elementRect.bottom : elementRect.bottom;

  return {
    top: offsetTop,
    left: offsetLeft,
    right: offsetRight,
    bottom: offsetBottom,
    x: offsetLeft,
    y: offsetTop,
    width: elementRect.width,
    height: elementRect.height
  };
}

/**
 * Get the scrollTop position
 *
 * @param {HTMLElement} target
 * @param {HTMLElement} offsetParent
 *
 * @returns {number}
 */
function getElementScrollY(target, offsetParent) {
  if (!target || !offsetParent) {
    return 0;
  }

  var targetRect = getOffsetBoundingClientRect(target, offsetParent);
  var targetTop = targetRect.top + getScrollTop();

  return Math.floor(targetTop);
}

function scrollTo(value) {
  scroll.top(scrollDoc(), value);
}

function getStyleComputedProperty(el) {
  if (el.nodeType !== 1) {
    return {};
  }

  return getComputedStyle(el);
}

function isFixed(el) {
  if (!el) {
    return false;
  }

  var nodeName = el.nodeName;


  if (nodeName === 'BODY' || nodeName === 'HTML') {
    return false;
  }

  if (getStyleComputedProperty(el).position === 'fixed') {
    return true;
  }

  return isFixed(el.parentNode);
}

var color = '#f04';
var colorRGB = [255, 0, 68];
var overlayColor = 'rgba(0, 0, 0, 0.5)';
var holeShadow = '0 0 15px rgba(0, 0, 0, 0.5)';
var beaconSize = 36;
var zIndex = 100;

var buttonReset = {
  backgroundColor: 'transparent',
  border: 0,
  borderRadius: 0,
  color: '#555',
  outline: 'none',
  lineHeight: 1,
  padding: 8,
  WebkitAppearance: 'none'
};

var width = 290;

if (window.innerWidth > 480) {
  width = 380;
} else if (window.innerWidth > 768) {
  width = 490;
}

var hole = {
  borderRadius: 4,
  position: 'absolute'
};

var overlay = {
  bottom: 0,
  left: 0,
  position: 'absolute',
  right: 0,
  top: 0,
  zIndex: zIndex
};

function getMergedStyles(styles) {
  var defaultStyles = {
    beacon: _extends({}, buttonReset, {
      display: 'inline-block',
      height: beaconSize,
      position: 'relative',
      width: beaconSize,
      zIndex: zIndex
    }),
    beaconInner: {
      animation: 'joyride-beacon-inner 1.2s infinite ease-in-out',
      backgroundColor: color,
      borderRadius: '50%',
      display: 'block',
      height: '50%',
      left: '50%',
      opacity: 0.7,
      position: 'absolute',
      top: '50%',
      transform: 'translate(-50%, -50%)',
      width: '50%'
    },
    beaconOuter: {
      animation: 'joyride-beacon-outer 1.2s infinite ease-in-out',
      backgroundColor: 'rgba(' + colorRGB.join(',') + ', 0.2)',
      border: '2px solid ' + color,
      borderRadius: '50%',
      boxSizing: 'border-box',
      display: 'block',
      height: '100%',
      left: 0,
      opacity: 0.9,
      position: 'absolute',
      top: 0,
      // transform: 'translateY(-50%)',
      transformOrigin: 'center',
      width: '100%'
    },
    tooltip: {
      borderRadius: 5,
      fontSize: 16,
      backgroundColor: '#fff',
      padding: 15,
      position: 'relative',
      width: width
    },
    tooltipContainer: {
      textAlign: 'center'
    },
    tooltipTitle: {
      fontSize: 18,
      margin: '0 0 10px 0'
    },
    tooltipContent: {
      padding: '20px 10px'
    },
    tooltipFooter: {
      alignItems: 'center',
      display: 'flex',
      justifyContent: 'space-between',
      marginTop: 15
    },
    buttonNext: _extends({}, buttonReset, {
      backgroundColor: color,
      borderRadius: 4,
      color: '#fff'
    }),
    buttonBack: _extends({}, buttonReset, {
      color: color,
      marginLeft: 'auto',
      marginRight: 5
    }),
    buttonClose: _extends({}, buttonReset, {
      height: 14,
      padding: 15,
      position: 'absolute',
      right: 0,
      top: 0,
      width: 14
    }),
    buttonSkip: _extends({}, buttonReset, {
      fontSize: 14
    }),
    overlay: _extends({}, overlay, {
      backgroundColor: overlayColor,
      mixBlendMode: 'hard-light'
    }),
    overlayLegacy: _extends({}, overlay),
    hole: _extends({}, hole, {
      backgroundColor: 'gray'
    }),
    holeLegacy: _extends({}, hole, {
      boxShadow: '0 0 0 9999px ' + overlayColor + ', ' + holeShadow
    })
  };

  return deepmerge(defaultStyles, styles || {});
}

var DEFAULTS = {
  locale: {
    back: 'Back',
    close: 'Close',
    last: 'Last',
    next: 'Next',
    skip: 'Skip'
  },
  step: {
    event: 'click',
    placement: 'bottom'
  },
  tooltipOptions: {
    disableFlip: true,
    offset: 15,
    wrapperOptions: {
      offset: -18,
      position: true
    }
  }
};

var validKeys$1 = ['allowClicksThruHole', 'beaconComponent', 'disableBeacon', 'disableOverlay', 'disableOverlayClicks', 'hideBackButton', 'holePadding', 'isFixed', 'locale', 'offsetParent', 'showProgress', 'showSkipButton', 'styles', 'tooltipComponent', 'tooltipOptions'];

/**
 * Validate if a step is valid
 *
 * @param {Object} step - A step object
 * @param {boolean} debug
 *
 * @returns {boolean} - True if the step is valid, false otherwise
 */
function validateStep(step) {
  var debug = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  if (!is.plainObject(step)) {
    log({
      title: 'validateStep',
      data: 'step must be an object',
      warn: true,
      debug: debug
    });
    return false;
  }

  if (!step.target) {
    log({
      title: 'validateStep',
      data: 'target is missing from the step',
      warn: true,
      debug: debug
    });
    return false;
  }

  return true;
}

/**
 * Validate if steps is valid
 *
 * @param {Array} steps - A steps array
 * @param {boolean} debug
 *
 * @returns {boolean} - True if the steps are valid, false otherwise
 */
function validateSteps(steps) {
  var debug = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  if (!is.array(steps)) {
    log({
      title: 'validateSteps',
      data: 'steps must be an array',
      warn: true,
      debug: debug
    });

    return false;
  }

  return steps.every(function (d) {
    return validateStep(d, debug);
  });
}

function getSharedProps(props) {
  return Object.keys(props).filter(function (d) {
    return validKeys$1.indexOf(d) !== -1;
  }).reduce(function (acc, i) {
    acc[i] = props[i];

    return acc;
  }, {});
}

function getMergedStep(step, props) {
  if (!step) return undefined;

  var tooltipOptions = deepmerge.all([DEFAULTS.tooltipOptions, props.tooltipOptions || {}, step.tooltipOptions || {}]);

  if (step.placementBeacon) {
    tooltipOptions.wrapperOptions.placement = step.placementBeacon;
  }

  return _extends({}, deepmerge.all([getSharedProps(props), DEFAULTS.step, step]), {
    locale: deepmerge(DEFAULTS.locale, props.locale || {}),
    tooltipOptions: tooltipOptions,
    styles: getMergedStyles(step.styles)
  });
}

var EVENTS = {
  TOUR_START: 'tour:start',
  STEP_BEFORE: 'step:before',
  BEACON: 'beacon',
  BEACON_TRIGGER: 'beacon:trigger',
  TOOLTIP: 'tooltip',
  TOOLTIP_SKIP: 'tooltip:skip',
  TOOLTIP_CLOSE: 'tooltip:close',
  OVERLAY_CLICK: 'overlay:click',
  HOLE_CLICK: 'hole:click',
  STEP_AFTER: 'step:after',
  TOUR_STATUS: 'tour:status',
  TARGET_NOT_FOUND: 'error:target_not_found',
  ERROR: 'error',
  TOUR_END: 'tour:end'
};

var JoyrideBeacon = function JoyrideBeacon(_ref) {
  var beaconComponent = _ref.beaconComponent,
      onClickOrHover = _ref.onClickOrHover,
      styles = _ref.styles;

  var props = {
    'aria-label': 'Open',
    onClick: onClickOrHover,
    onMouseEnter: onClickOrHover,
    title: 'Open'
  };
  var component = void 0;

  if (beaconComponent) {
    if (React.isValidElement(beaconComponent)) {
      component = React.cloneElement(beaconComponent, props);
    } else {
      component = beaconComponent(props);
    }
  } else {
    component = React.createElement(
      'button',
      _extends({
        key: 'JoyrideBeacon',
        className: 'joyride-beacon',
        style: styles.beacon
      }, props),
      React.createElement('span', { style: styles.beaconInner }),
      React.createElement('span', { style: styles.beaconOuter })
    );
  }

  return component;
};

JoyrideBeacon.propTypes = {
  beaconComponent: PropTypes.oneOfType([PropTypes.func, PropTypes.element]),
  onClickOrHover: PropTypes.func.isRequired,
  styles: PropTypes.object.isRequired
};

var JoyrideHole = function JoyrideHole(_ref) {
  var styles = _ref.styles;
  return React.createElement('div', {
    key: 'JoyrideHole',
    className: 'joyride-hole',
    style: styles
  });
};

JoyrideHole.propTypes = {
  styles: PropTypes.object.isRequired
};

var Overlay = function (_React$Component) {
  inherits(Overlay, _React$Component);

  function Overlay() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, Overlay);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Overlay.__proto__ || Object.getPrototypeOf(Overlay)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      mouseOverHole: false
    }, _this.handleMouseMove = function (e) {
      var mouseOverHole = _this.state.mouseOverHole;
      var _this$stylesHole = _this.stylesHole,
          height = _this$stylesHole.height,
          left = _this$stylesHole.left,
          position = _this$stylesHole.position,
          top = _this$stylesHole.top,
          width = _this$stylesHole.width;


      var offsetY = position === 'fixed' ? e.clientY : e.pageY;
      var offsetX = position === 'fixed' ? e.clientX : e.pageX;
      var inHoleHeight = offsetY >= top && offsetY <= top + height;
      var inHoleWidth = offsetX >= left && offsetX <= left + width;
      var inHole = inHoleWidth && inHoleHeight;

      if (inHole !== mouseOverHole) {
        _this.setState({ mouseOverHole: inHole });
      }
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(Overlay, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _props = this.props,
          allowClicksThruHole = _props.allowClicksThruHole,
          disableOverlay = _props.disableOverlay;


      if (!disableOverlay && allowClicksThruHole) {
        document.addEventListener('mousemove', this.handleMouseMove, false);
      }
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var allowClicksThruHole = nextProps.allowClicksThruHole,
          disableOverlay = nextProps.disableOverlay;

      var _treeChanges = treeChanges(this.props, nextProps),
          changed = _treeChanges.changed;

      if (changed('disableOverlay')) {
        if (!disableOverlay && allowClicksThruHole) {
          document.addEventListener('mousemove', this.handleMouseMove, false);
        } else {
          document.removeEventListener('mousemove', this.handleMouseMove, false);
        }
      }

      if (changed('allowClicksThruHole')) {
        if (allowClicksThruHole) {
          document.addEventListener('mousemove', this.handleMouseMove, false);
        } else {
          document.removeEventListener('mousemove', this.handleMouseMove, false);
        }
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      document.removeEventListener('mousemove', this.handleMouseMove, false);
    }
  }, {
    key: 'render',
    value: function render() {
      var _props2 = this.props,
          allowClicksThruHole = _props2.allowClicksThruHole,
          disableOverlay = _props2.disableOverlay,
          holePadding = _props2.holePadding,
          lifecycle = _props2.lifecycle,
          offsetParent = _props2.offsetParent,
          onClickOverlay = _props2.onClickOverlay,
          target = _props2.target,
          styles = _props2.styles;

      var output = {};

      if (disableOverlay || lifecycle !== LIFECYCLE.TOOLTIP) {
        return null;
      }

      var stylesOverlay = _extends({
        cursor: disableOverlay ? 'default' : 'pointer',
        height: document.body.clientHeight,
        pointerEvents: this.state.mouseOverHole ? 'none' : 'auto'
      }, isLegacy() ? styles.overlayLegacy : styles.overlay);

      return React.createElement(
        'div',
        {
          className: 'joyride-overlay',
          style: stylesOverlay,
          onClick: onClickOverlay
        },
        React.createElement(JoyrideHole, {
          allowClicksThruHole: allowClicksThruHole,
          holePadding: holePadding,
          offsetParent: offsetParent,
          target: target,
          styles: this.stylesHole
        }),
        output.hole
      );
    }
  }, {
    key: 'stylesHole',
    get: function get$$1() {
      var _props3 = this.props,
          allowClicksThruHole = _props3.allowClicksThruHole,
          holePadding = _props3.holePadding,
          offsetParent = _props3.offsetParent,
          target = _props3.target,
          styles = _props3.styles;


      var rect = getOffsetBoundingClientRect(getElement(target), getElement(offsetParent));
      var isFixedTarget = isFixed(target);

      return _extends({}, isLegacy() ? styles.holeLegacy : styles.hole, {
        height: Math.round(rect.height + holePadding * 2),
        left: Math.round(rect.left - holePadding),
        pointerEvents: allowClicksThruHole ? 'none' : 'auto',
        position: isFixedTarget ? 'fixed' : 'absolute',
        top: Math.round(rect.top - (isFixedTarget ? 0 : document.body.getBoundingClientRect().top) - holePadding),
        width: Math.round(rect.width + holePadding * 2)
      });
    }
  }]);
  return Overlay;
}(React.Component);

Overlay.propTypes = {
  allowClicksThruHole: PropTypes.bool.isRequired,
  disableOverlay: PropTypes.bool.isRequired,
  holePadding: PropTypes.number,
  lifecycle: PropTypes.string.isRequired,
  offsetParent: PropTypes.oneOfType([PropTypes.object, PropTypes.string]).isRequired,
  onClickOverlay: PropTypes.func.isRequired,
  styles: PropTypes.object.isRequired,
  target: PropTypes.oneOfType([PropTypes.object, PropTypes.string]).isRequired
};

var CloseBtn = function CloseBtn(_ref) {
  var styles = _ref.styles,
      props = objectWithoutProperties(_ref, ['styles']);
  var color = styles.color,
      height = styles.height,
      width = styles.width,
      style = objectWithoutProperties(styles, ['color', 'height', 'width']);


  return React.createElement(
    'button',
    _extends({ style: style }, props),
    React.createElement(
      'svg',
      {
        width: width + 'px',
        height: height + 'px',
        viewBox: '0 0 18 18',
        version: '1.1',
        xmlns: 'http://www.w3.org/2000/svg',
        preserveAspectRatio: 'xMidYMid'
      },
      React.createElement(
        'g',
        null,
        React.createElement('path', {
          d: 'M8.13911129,9.00268191 L0.171521827,17.0258467 C-0.0498027049,17.248715 -0.0498027049,17.6098394 0.171521827,17.8327545 C0.28204354,17.9443526 0.427188206,17.9998706 0.572051765,17.9998706 C0.71714958,17.9998706 0.862013139,17.9443526 0.972581703,17.8327545 L9.0000937,9.74924618 L17.0276057,17.8327545 C17.1384085,17.9443526 17.2832721,17.9998706 17.4281356,17.9998706 C17.5729992,17.9998706 17.718097,17.9443526 17.8286656,17.8327545 C18.0499901,17.6098862 18.0499901,17.2487618 17.8286656,17.0258467 L9.86135722,9.00268191 L17.8340066,0.973848225 C18.0553311,0.750979934 18.0553311,0.389855532 17.8340066,0.16694039 C17.6126821,-0.0556467968 17.254037,-0.0556467968 17.0329467,0.16694039 L9.00042166,8.25611765 L0.967006424,0.167268345 C0.745681892,-0.0553188426 0.387317931,-0.0553188426 0.165993399,0.167268345 C-0.0553311331,0.390136635 -0.0553311331,0.751261038 0.165993399,0.974176179 L8.13920499,9.00268191 L8.13911129,9.00268191 Z',
          fill: color
        })
      )
    )
  );
};

CloseBtn.propTypes = {
  handleClick: PropTypes.func.isRequired,
  styles: PropTypes.object.isRequired
};

var JoyrideTooltipContainer = function JoyrideTooltipContainer(_ref) {
  var continuous = _ref.continuous,
      backProps = _ref.backProps,
      closeProps = _ref.closeProps,
      primaryProps = _ref.primaryProps,
      skipProps = _ref.skipProps,
      index = _ref.index,
      isLastStep = _ref.isLastStep,
      step = _ref.step;
  var content = step.content,
      hideBackButton = step.hideBackButton,
      locale = step.locale,
      showSkipButton = step.showSkipButton,
      title = step.title,
      styles = step.styles;
  var back = locale.back,
      close = locale.close,
      last = locale.last,
      next = locale.next,
      skip = locale.skip;

  var output = {
    primary: close
  };

  if (continuous) {
    if (isLastStep) {
      output.primary = last;
    } else {
      output.primary = next;
    }
  }

  if (showSkipButton && !isLastStep) {
    output.skip = React.createElement(
      'button',
      _extends({ style: styles.buttonSkip }, skipProps),
      skip
    );
  }

  if (!hideBackButton && index > 0) {
    output.back = React.createElement(
      'button',
      _extends({ style: styles.buttonBack }, backProps),
      back
    );
  }

  output.close = React.createElement(CloseBtn, _extends({}, closeProps, { styles: styles.buttonClose }));

  return React.createElement(
    'div',
    {
      key: 'JoyrideTooltip',
      style: styles.tooltip
    },
    React.createElement(
      'div',
      { style: styles.tooltipContainer },
      output.close,
      title && React.createElement(
        'h4',
        { style: styles.tooltipTitle },
        title
      ),
      content && React.createElement(
        'div',
        { style: styles.tooltipContent },
        content
      )
    ),
    React.createElement(
      'div',
      { style: styles.tooltipFooter },
      output.skip,
      output.back,
      React.createElement(
        'button',
        _extends({ style: styles.buttonNext }, primaryProps),
        output.primary
      )
    )
  );
};

JoyrideTooltipContainer.propTypes = {
  backProps: PropTypes.object.isRequired,
  closeProps: PropTypes.object.isRequired,
  continuous: PropTypes.bool.isRequired,
  index: PropTypes.number.isRequired,
  isLastStep: PropTypes.bool.isRequired,
  primaryProps: PropTypes.object.isRequired,
  skipProps: PropTypes.object.isRequired,
  step: PropTypes.object.isRequired
};

var JoyrideTooltip = function (_React$Component) {
  inherits(JoyrideTooltip, _React$Component);

  function JoyrideTooltip() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, JoyrideTooltip);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = JoyrideTooltip.__proto__ || Object.getPrototypeOf(JoyrideTooltip)).call.apply(_ref, [this].concat(args))), _this), _this.handleClickBack = function (e) {
      e.preventDefault();

      _this.props.helpers.prev();
    }, _this.handleClickClose = function (e) {
      e.preventDefault();

      _this.props.helpers.close();
    }, _this.handleClickPrimary = function (e) {
      e.preventDefault();
      var continuous = _this.props.continuous;


      if (!continuous) {
        _this.props.helpers.close();
        return;
      }

      _this.props.helpers.next();
    }, _this.handleClickSkip = function (e) {
      e.preventDefault();

      _this.props.helpers.skip();
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(JoyrideTooltip, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          continuous = _props.continuous,
          index = _props.index,
          isLastStep = _props.isLastStep,
          step = _props.step;
      var content = step.content,
          locale = step.locale,
          title = step.title,
          tooltipComponent = step.tooltipComponent;
      var back = locale.back,
          close = locale.close,
          last = locale.last,
          next = locale.next,
          skip = locale.skip;

      var primaryText = continuous ? next : close;

      if (isLastStep) {
        primaryText = last;
      }

      var component = void 0;
      var props = {
        backProps: { 'aria-label': back, onClick: this.handleClickBack, role: 'button', title: back },
        closeProps: { 'aria-label': close, onClick: this.handleClickClose, role: 'button', title: close },
        primaryProps: { 'aria-label': primaryText, onClick: this.handleClickPrimary, role: 'button', title: primaryText },
        skipProps: { 'aria-label': skip, onClick: this.handleClickSkip, role: 'button', title: skip }
      };

      if (tooltipComponent) {
        var richProps = _extends({}, props, {
          content: content,
          locale: locale,
          title: title
        });

        if (React.isValidElement(tooltipComponent)) {
          component = React.cloneElement(tooltipComponent, richProps);
        } else {
          component = tooltipComponent(richProps);
        }
      } else {
        component = React.createElement(JoyrideTooltipContainer, _extends({
          continuous: continuous,
          index: index,
          isLastStep: isLastStep,
          step: step
        }, props));
      }

      return component;
    }
  }]);
  return JoyrideTooltip;
}(React.Component);

JoyrideTooltip.propTypes = {
  continuous: PropTypes.bool.isRequired,
  helpers: PropTypes.object.isRequired,
  index: PropTypes.number.isRequired,
  isLastStep: PropTypes.bool.isRequired,
  step: PropTypes.object.isRequired
};

var JoyrideStep = function (_React$Component) {
  inherits(JoyrideStep, _React$Component);

  function JoyrideStep() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, JoyrideStep);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = JoyrideStep.__proto__ || Object.getPrototypeOf(JoyrideStep)).call.apply(_ref, [this].concat(args))), _this), _this.handleClickHoverBeacon = function (e) {
      var _this$props = _this.props,
          step = _this$props.step,
          update = _this$props.update;


      if (e.type === 'mouseenter' && step.event !== 'hover') {
        return;
      }

      update({ lifecycle: LIFECYCLE.TOOLTIP });
    }, _this.handleClickOverlay = function () {
      _this.props.helpers.close();
    }, _this.getPopper = function (popper, type) {
      var getPopper = _this.props.getPopper;


      if (type === 'wrapper') {
        _this.beaconPopper = popper;
      } else {
        _this.tooltipPopper = popper;
      }

      getPopper(popper, type);
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(JoyrideStep, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _props = this.props,
          debug = _props.debug,
          lifecycle = _props.lifecycle;


      log({
        title: 'step:' + lifecycle,
        data: [{ key: 'props', value: this.props }],
        debug: debug
      });
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var _props2 = this.props,
          action = _props2.action,
          continuous = _props2.continuous,
          debug = _props2.debug,
          index = _props2.index,
          lifecycle = _props2.lifecycle,
          step = _props2.step,
          update = _props2.update;

      var _treeChanges = treeChanges(this.props, nextProps),
          changed = _treeChanges.changed,
          changedFrom = _treeChanges.changedFrom,
          changedTo = _treeChanges.changedTo;

      var skipBeacon = continuous && action !== ACTIONS.CLOSE && (index > 0 || action === ACTIONS.PREV);

      if (changedFrom('lifecycle', LIFECYCLE.INIT, LIFECYCLE.READY)) {
        update({ lifecycle: step.disableBeacon || skipBeacon ? LIFECYCLE.TOOLTIP : LIFECYCLE.BEACON });
      }

      if (changed('index')) {
        log({
          title: 'step:' + lifecycle,
          data: [{ key: 'props', value: this.props }],
          debug: debug
        });
      }

      if (changedTo(LIFECYCLE.INIT)) {
        delete this.beaconPopper;
        delete this.tooltipPopper;
      }
    }

    /**
     * Beacon click/hover event listener
     *
     * @param {Event} e
     */

  }, {
    key: 'render',
    value: function render() {
      var _props3 = this.props,
          continuous = _props3.continuous,
          controlled = _props3.controlled,
          debug = _props3.debug,
          helpers = _props3.helpers,
          index = _props3.index,
          lifecycle = _props3.lifecycle,
          size = _props3.size,
          step = _props3.step;

      var target = getElement(step.target);

      if (!validateStep(step) || !is.domElement(target)) {
        return null;
      }

      return React.createElement(
        'div',
        { key: 'JoyrideStep-' + index, className: 'joyride-step' },
        React.createElement(
          ReactTooltips,
          _extends({
            component: React.createElement(JoyrideTooltip, {
              continuous: continuous,
              controlled: controlled,
              helpers: helpers,
              index: index,
              isLastStep: index + 1 === size,
              step: step
            }),
            debug: debug,
            getPopper: this.getPopper,
            id: 'react-joyride:' + index,
            positionFixed: step.isFixed || isFixed(target),
            open: this.open,
            placement: step.placement,
            target: step.target
          }, step.tooltipOptions),
          React.createElement(JoyrideBeacon, { beaconComponent: step.beaconComponent, onClickOrHover: this.handleClickHoverBeacon, styles: step.styles })
        ),
        React.createElement(Overlay, _extends({}, step, {
          onClickOverlay: this.handleClickOverlay,
          lifecycle: lifecycle
        }))
      );
    }
  }, {
    key: 'open',
    get: function get$$1() {
      var _props4 = this.props,
          step = _props4.step,
          lifecycle = _props4.lifecycle;


      return !!(step.disableBeacon || lifecycle === LIFECYCLE.TOOLTIP);
    }
  }]);
  return JoyrideStep;
}(React.Component);

JoyrideStep.propTypes = {
  action: PropTypes.string.isRequired,
  continuous: PropTypes.bool.isRequired,
  controlled: PropTypes.bool.isRequired,
  debug: PropTypes.bool.isRequired,
  getPopper: PropTypes.func.isRequired,
  helpers: PropTypes.object.isRequired,
  index: PropTypes.number.isRequired,
  lifecycle: PropTypes.string.isRequired,
  size: PropTypes.number.isRequired,
  status: PropTypes.string.isRequired,
  step: PropTypes.shape({
    allowClicksThruHole: PropTypes.bool,
    beaconComponent: PropTypes.oneOfType([PropTypes.func, PropTypes.element]),
    content: isRequiredIf(PropTypes.node, function (props) {
      return !props.tooltipComponent && !props.title;
    }),
    disableBeacon: PropTypes.bool,
    disableOverlay: PropTypes.bool,
    disableOverlayClicks: PropTypes.bool,
    event: PropTypes.string,
    hideBackButton: PropTypes.bool,
    holePadding: PropTypes.number,
    isFixed: PropTypes.bool,
    locale: PropTypes.object,
    offsetParent: PropTypes.oneOfType([PropTypes.object, PropTypes.string]).isRequired,
    placement: PropTypes.oneOf(['top', 'top-start', 'top-end', 'bottom', 'bottom-start', 'bottom-end', 'left', 'left-start', 'left-end', 'right', 'right-start', 'right-end', 'auto', 'center']),
    target: PropTypes.oneOfType([PropTypes.object, PropTypes.string]).isRequired,
    styles: PropTypes.object,
    title: PropTypes.node,
    tooltipComponent: isRequiredIf(PropTypes.oneOfType([PropTypes.func, PropTypes.element]), function (props) {
      return !props.content && !props.title;
    }),
    tooltipOptions: PropTypes.shape({
      offset: PropTypes.number
    })
  }).isRequired,
  update: PropTypes.func.isRequired
};

var Joyride = function (_React$Component) {
  inherits(Joyride, _React$Component);

  function Joyride(props) {
    classCallCheck(this, Joyride);

    var _this = possibleConstructorReturn(this, (Joyride.__proto__ || Object.getPrototypeOf(Joyride)).call(this, props));

    _this.handleKeyboardNavigation = function (e) {
      var lifecycle = _this.state.lifecycle;

      var intKey = window.Event ? e.which : e.keyCode;

      if (lifecycle === LIFECYCLE.TOOLTIP) {
        if (intKey === 27) {
          _this.store.close();
        }
      }
    };

    _this.syncState = function (state) {
      _this.setState(state);
    };

    _this.getPopper = function (popper, type) {
      if (type === 'wrapper') {
        _this.beaconPopper = popper;
      } else {
        _this.tooltipPopper = popper;
      }

      if (_this.beaconPopper && _this.tooltipPopper) {
        var action = _this.state.action;


        _this.store.update({
          action: action === ACTIONS.CLOSE ? ACTIONS.CLOSE : action,
          lifecycle: LIFECYCLE.READY
        });
      }
    };

    _this.store = new createState(_extends({}, props, {
      controlled: props.run && is.number(props.stepIndex)
    }));
    _this.state = _this.store.getState();
    _this.helpers = _this.store.getHelpers();
    return _this;
  }

  createClass(Joyride, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _props = this.props,
          debug = _props.debug,
          disableCloseOnEsc = _props.disableCloseOnEsc,
          run = _props.run,
          steps = _props.steps;
      var start = this.store.start;


      log({
        title: 'init',
        data: [{ key: 'props', value: this.props }, { key: 'state', value: this.state }],
        debug: debug
      });

      // Sync the store to this component state.
      this.store.addListener(this.syncState);

      if (validateSteps(steps, debug) && run) {
        start();
      }

      /* istanbul ignore else */
      if (!disableCloseOnEsc) {
        document.body.addEventListener('keydown', this.handleKeyboardNavigation);
      }

      window.addEventListener('touchstart', function setHasTouch() {
        // Remove event listener once fired, otherwise it'll kill scrolling
        // performance
        window.removeEventListener('touchstart', setHasTouch);
      }, false);
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var _state = this.state,
          action = _state.action,
          status = _state.status;
      var _props2 = this.props,
          steps = _props2.steps,
          stepIndex = _props2.stepIndex;
      var debug = nextProps.debug,
          run = nextProps.run,
          nextSteps = nextProps.steps,
          nextStepIndex = nextProps.stepIndex;
      var _store = this.store,
          setSteps = _store.setSteps,
          start = _store.start,
          stop = _store.stop,
          update = _store.update;

      var _treeChanges = treeChanges(this.props, nextProps),
          changed = _treeChanges.changed;

      var diffProps = deep.diff(this.props, nextProps);

      if (diffProps) {
        log({
          title: 'props',
          data: [{ key: 'nextProps', value: nextProps }, { key: 'changed', value: diffProps }],
          debug: debug
        });

        var stepsChanged = deep.diff(nextSteps, steps);
        var stepIndexChanged = is.number(nextStepIndex) && changed('stepIndex');
        var shouldStart = false;

        /* istanbul ignore else */
        if (changed('run')) {
          if (run) {
            shouldStart = true;
          } else {
            stop();
          }
        }

        if (stepsChanged) {
          if (validateSteps(nextSteps, debug)) {
            setSteps(nextSteps);
          } else {
            console.warn('Steps are not valid', nextSteps); //eslint-disable-line no-console
          }
        }

        /* istanbul ignore else */
        if (stepIndexChanged) {
          var nextAction = stepIndex < nextStepIndex ? ACTIONS.NEXT : ACTIONS.PREV;

          if ([STATUS.FINISHED, STATUS.SKIPPED].indexOf(status) !== -1) {
            update({
              action: ACTIONS.START,
              index: nextStepIndex,
              lifecycle: LIFECYCLE.INIT,
              status: run && STATUS.RUNNING
            });
          } else {
            update({
              action: action === ACTIONS.CLOSE ? ACTIONS.CLOSE : nextAction,
              index: nextStepIndex,
              lifecycle: LIFECYCLE.INIT
            });
          }
        }

        if (shouldStart) {
          start();
        }
      }
    }
  }, {
    key: 'componentWillUpdate',
    value: function componentWillUpdate(nextProps, nextState) {
      var index = this.state.index;
      var stepIndex = nextProps.stepIndex,
          steps = nextProps.steps;

      var _treeChanges2 = treeChanges(this.state, nextState),
          changed = _treeChanges2.changed,
          changedFrom = _treeChanges2.changedFrom;

      var step = getMergedStep(steps[index], nextProps);

      var isControlled = is.number(stepIndex);
      var hasChangedIndex = changed('index') && changedFrom('lifecycle', LIFECYCLE.TOOLTIP, LIFECYCLE.INIT);
      var isAfterAction = changed('action') && [ACTIONS.NEXT, ACTIONS.PREV, ACTIONS.SKIP, ACTIONS.CLOSE].indexOf(nextState.action) !== -1;

      if (isControlled && isAfterAction || hasChangedIndex) {
        this.callback(_extends({}, nextState, {
          lifecycle: LIFECYCLE.COMPLETE,
          step: step,
          type: EVENTS.STEP_AFTER
        }));
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps, prevState) {
      var _state2 = this.state,
          action = _state2.action,
          index = _state2.index;
      var _props3 = this.props,
          steps = _props3.steps,
          stepIndex = _props3.stepIndex;

      var step = getMergedStep(steps[index], this.props);

      var _treeChanges3 = treeChanges(prevState, this.state),
          changed = _treeChanges3.changed,
          changedFrom = _treeChanges3.changedFrom,
          changedTo = _treeChanges3.changedTo;

      var diffState = deep.diff(prevState, this.state);
      var isControlled = is.number(stepIndex);

      if (diffState) {
        log({
          title: 'state',
          data: [{ key: 'state', value: this.state }, { key: 'changed', value: diffState }, { key: 'step', value: step }],
          debug: this.props.debug
        });

        if (changed('status')) {
          var type = EVENTS.TOUR_STATUS;

          if (changedTo('status', STATUS.FINISHED) || changedTo('status', STATUS.SKIPPED)) {
            type = EVENTS.TOUR_END;
          } else if (changedFrom('status', STATUS.READY, STATUS.RUNNING)) {
            type = EVENTS.TOUR_START;
          }

          this.callback(_extends({}, this.state, {
            step: step,
            type: type
          }));
        }

        // There's a step to use, but there's no target in the DOM
        if (step) {
          var hasRenderedTarget = Boolean(getElement(step.target));

          if (hasRenderedTarget) {
            if (changedFrom('status', STATUS.READY, STATUS.RUNNING) || changed('index')) {
              this.callback(_extends({}, this.state, {
                step: step,
                type: EVENTS.STEP_BEFORE
              }));
            }
          } else {
            console.warn('Target not mounted', step); //eslint-disable-line no-console
            this.callback(_extends({}, this.state, {
              type: EVENTS.TARGET_NOT_FOUND,
              step: step
            }));

            if (!isControlled) {
              this.store.update({ index: index + ([ACTIONS.PREV].indexOf(action) !== -1 ? -1 : 1) });
            }
          }
        }

        /* istanbul ignore else */
        if (changedTo('lifecycle', LIFECYCLE.BEACON)) {
          this.callback(_extends({}, this.state, {
            step: step,
            type: EVENTS.BEACON
          }));
        }

        if (changedTo('lifecycle', LIFECYCLE.TOOLTIP)) {
          this.callback(_extends({}, this.state, {
            step: step,
            type: EVENTS.TOOLTIP
          }));
        }

        if (changedTo('lifecycle', LIFECYCLE.INIT)) {
          delete this.beaconPopper;
          delete this.tooltipPopper;
        }
      }

      this.scrollToStep(prevState);
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      var disableCloseOnEsc = this.props.disableCloseOnEsc;

      /* istanbul ignore else */

      if (!disableCloseOnEsc) {
        document.body.removeEventListener('keydown', this.handleKeyboardNavigation);
      }
    }

    /**
     * Trigger the callback.
     *
     * @private
     * @param {Object} data
     */

  }, {
    key: 'callback',
    value: function callback(data) {
      var callback = this.props.callback;

      /* istanbul ignore else */

      if (is.function(callback)) {
        callback(data);
      }
    }
  }, {
    key: 'scrollToStep',
    value: function scrollToStep(prevState) {
      var _state3 = this.state,
          index = _state3.index,
          lifecycle = _state3.lifecycle,
          status = _state3.status;
      var _props4 = this.props,
          debug = _props4.debug,
          disableScrollToSteps = _props4.disableScrollToSteps,
          scrollToFirstStep = _props4.scrollToFirstStep,
          scrollOffset = _props4.scrollOffset,
          steps = _props4.steps;

      var step = getMergedStep(steps[index], this.props);

      if (step) {
        var target = getElement(step.target);

        var shouldScroll = step && !disableScrollToSteps && (!step.isFixed || !isFixed(target)) // fixed steps don't need to scroll
        && prevState.lifecycle !== lifecycle && [LIFECYCLE.BEACON, LIFECYCLE.TOOLTIP].indexOf(lifecycle) !== -1 && (scrollToFirstStep || prevState.index !== index);

        if (status === STATUS.RUNNING && shouldScroll) {
          var scrollY = 0;

          log({
            title: 'scrollToStep',
            data: [{ key: 'index', value: index }, { key: 'lifecycle', value: lifecycle }, { key: 'status', value: status }],
            debug: debug
          });

          scrollY = Math.floor(getElementScrollY(target, getElement(step.offsetParent)));

          if (lifecycle === LIFECYCLE.BEACON && this.beaconPopper) {
            var _beaconPopper = this.beaconPopper,
                placement = _beaconPopper.placement,
                popper = _beaconPopper.popper;

            if (placement === 'top') {
              scrollY = Math.floor(popper.top);
            }
          } else if (lifecycle === LIFECYCLE.TOOLTIP && this.tooltipPopper) {
            var _tooltipPopper = this.tooltipPopper,
                _placement = _tooltipPopper.placement,
                _popper = _tooltipPopper.popper;


            if (['top', 'right'].indexOf(_placement) !== -1) {
              scrollY = Math.floor(_popper.top);
            }
          }

          if (status === STATUS.RUNNING && shouldScroll && scrollY >= 0) {
            scrollTo(scrollY - scrollOffset);
          }
        }
      }
    }

    /**
     * Keydown event listener
     *
     * @private
     * @param {Event} e - Keyboard event
     */


    /**
     * Sync the store with the component's state
     *
     * @param {Object} state
     */

  }, {
    key: 'render',
    value: function render() {
      var _state4 = this.state,
          index = _state4.index,
          status = _state4.status;
      var _props5 = this.props,
          continuous = _props5.continuous,
          debug = _props5.debug,
          steps = _props5.steps;

      var step = getMergedStep(steps[index], this.props);
      var output = void 0;

      if (status === STATUS.RUNNING && step) {
        output = React.createElement(JoyrideStep, _extends({}, this.state, {
          continuous: continuous,
          debug: debug,
          helpers: this.helpers,
          locale: this.locale,
          getPopper: this.getPopper,
          step: step,
          update: this.store.update
        }));
      }

      return React.createElement(
        'div',
        { className: 'joyride' },
        output
      );
    }
  }]);
  return Joyride;
}(React.Component);

Joyride.propTypes = {
  allowClicksThruHole: PropTypes.bool,
  beaconComponent: PropTypes.oneOfType([PropTypes.func, PropTypes.element]),
  callback: PropTypes.func,
  continuous: PropTypes.bool,
  debug: PropTypes.bool,
  disableBeacon: PropTypes.bool,
  disableCloseOnEsc: PropTypes.bool,
  disableOverlay: PropTypes.bool,
  disableOverlayClicks: PropTypes.bool,
  disableScrollToSteps: PropTypes.bool,
  hideBackButton: PropTypes.bool,
  holePadding: PropTypes.number,
  locale: PropTypes.object,
  offsetParent: PropTypes.oneOfType([PropTypes.object, PropTypes.string]),
  run: PropTypes.bool,
  scrollOffset: PropTypes.number,
  scrollToFirstStep: PropTypes.bool,
  showProgress: PropTypes.bool,
  showSkipButton: PropTypes.bool,
  stepIndex: PropTypes.number,
  steps: PropTypes.array,
  tooltipComponent: PropTypes.oneOfType([PropTypes.func, PropTypes.element]),
  tooltipOptions: PropTypes.shape({
    offset: PropTypes.number
  })
};
Joyride.defaultProps = {
  allowClicksThruHole: false,
  continuous: false,
  debug: false,
  disableBeacon: false,
  disableCloseOnEsc: false,
  disableOverlay: false,
  disableOverlayClicks: false,
  disableScrollToSteps: false,
  hideBackButton: false,
  holePadding: 10,
  offsetParent: 'body',
  run: false,
  scrollOffset: 20,
  scrollToFirstStep: false,
  showSkipButton: false,
  showProgress: false,
  steps: []
};

exports.default = Joyride;
